@@ -8,14 +8,15 @@ use Drupal\Component\Utility\Variable;
 use Drupal\Component\Utility\Xss;
 use Drupal\Core\Access\AccessResultInterface;
 use Drupal\Core\Cache\Cache;
+use Drupal\Core\Cache\CacheableDependencyInterface;
 use Drupal\Core\Cache\CacheableMetadata;
-use Drupal\Core\Controller\ControllerResolverInterface;
 use Drupal\Core\Form\FormHelper;
 use Drupal\Core\Render\Element\RenderCallbackInterface;
 use Drupal\Core\Security\TrustedCallbackInterface;
 use Drupal\Core\Security\DoTrustedCallbackTrait;
 use Drupal\Core\Theme\ThemeManagerInterface;
 use Drupal\Core\Utility\CallableResolver;
+use Drupal\Core\Utility\FiberResumeType;
 use Symfony\Component\HttpFoundation\RequestStack;
 
 /**
@@ -24,41 +25,6 @@ use Symfony\Component\HttpFoundation\RequestStack;
 class Renderer implements RendererInterface {
   use DoTrustedCallbackTrait;
 
-  /**
-   * The theme manager.
-   *
-   * @var \Drupal\Core\Theme\ThemeManagerInterface
-   */
-  protected $theme;
-
-  /**
-   * The callable resolver.
-   *
-   * @var \Drupal\Core\Utility\CallableResolver
-   */
-  protected CallableResolver $callableResolver;
-
-  /**
-   * The element info.
-   *
-   * @var \Drupal\Core\Render\ElementInfoManagerInterface
-   */
-  protected $elementInfo;
-
-  /**
-   * The placeholder generator.
-   *
-   * @var \Drupal\Core\Render\PlaceholderGeneratorInterface
-   */
-  protected $placeholderGenerator;
-
-  /**
-   * The render cache service.
-   *
-   * @var \Drupal\Core\Render\RenderCacheInterface
-   */
-  protected $renderCache;
-
   /**
    * The renderer configuration array.
    *
@@ -73,13 +39,6 @@ class Renderer implements RendererInterface {
    */
   protected $isRenderingRoot = FALSE;
 
-  /**
-   * The request stack.
-   *
-   * @var \Symfony\Component\HttpFoundation\RequestStack
-   */
-  protected $requestStack;
-
   /**
    * The render context collection.
    *
@@ -100,36 +59,34 @@ class Renderer implements RendererInterface {
   /**
    * Constructs a new Renderer.
    *
-   * @param \Drupal\Core\Utility\CallableResolver|\Drupal\Core\Controller\ControllerResolverInterface $callable_resolver
+   * @param \Drupal\Core\Utility\CallableResolver $callableResolver
    *   The callable resolver.
    * @param \Drupal\Core\Theme\ThemeManagerInterface $theme
    *   The theme manager.
-   * @param \Drupal\Core\Render\ElementInfoManagerInterface $element_info
+   * @param \Drupal\Core\Render\ElementInfoManagerInterface $elementInfo
    *   The element info.
-   * @param \Drupal\Core\Render\PlaceholderGeneratorInterface $placeholder_generator
+   * @param \Drupal\Core\Render\PlaceholderGeneratorInterface $placeholderGenerator
    *   The placeholder generator.
-   * @param \Drupal\Core\Render\RenderCacheInterface $render_cache
+   * @param \Drupal\Core\Render\RenderCacheInterface $renderCache
    *   The render cache service.
-   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
+   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
    *   The request stack.
    * @param array $renderer_config
    *   The renderer configuration array.
    */
-  public function __construct(ControllerResolverInterface|CallableResolver $callable_resolver, ThemeManagerInterface $theme, ElementInfoManagerInterface $element_info, PlaceholderGeneratorInterface $placeholder_generator, RenderCacheInterface $render_cache, RequestStack $request_stack, array $renderer_config) {
-    if ($callable_resolver instanceof ControllerResolverInterface) {
-      @trigger_error('Calling ' . __METHOD__ . '() with an argument of ControllerResolverInterface is deprecated in drupal:10.2.0 and is removed in drupal:11.0.0. Use \Drupal\Core\Utility\CallableResolver instead. See https://www.drupal.org/node/3369969', E_USER_DEPRECATED);
-      $callable_resolver = \Drupal::service('callable_resolver');
-    }
-    $this->callableResolver = $callable_resolver;
-    $this->theme = $theme;
-    $this->elementInfo = $element_info;
-    $this->placeholderGenerator = $placeholder_generator;
-    $this->renderCache = $render_cache;
+  public function __construct(
+    protected CallableResolver $callableResolver,
+    protected ThemeManagerInterface $theme,
+    protected ElementInfoManagerInterface $elementInfo,
+    protected PlaceholderGeneratorInterface $placeholderGenerator,
+    protected RenderCacheInterface $renderCache,
+    protected RequestStack $requestStack,
+    array $renderer_config,
+  ) {
     if (!isset($renderer_config['debug'])) {
       $renderer_config['debug'] = FALSE;
     }
     $this->rendererConfig = $renderer_config;
-    $this->requestStack = $request_stack;
 
     // Initialize the context collection if needed.
     if (!isset(static::$contextCollection)) {
@@ -141,6 +98,9 @@ class Renderer implements RendererInterface {
    * {@inheritdoc}
    */
   public function renderRoot(&$elements) {
+    if (!$elements) {
+      return '';
+    }
     // Disallow calling ::renderRoot() from within another ::renderRoot() call.
     if ($this->isRenderingRoot) {
       $this->isRenderingRoot = FALSE;
@@ -149,21 +109,39 @@ class Renderer implements RendererInterface {
 
     // Render in its own render context.
     $this->isRenderingRoot = TRUE;
-    $output = $this->executeInRenderContext(new RenderContext(), function () use (&$elements) {
-      return $this->render($elements, TRUE);
-    });
-    $this->isRenderingRoot = FALSE;
+    try {
+      $output = $this->renderInIsolation($elements);
+    }
+    // Since #pre_render, #post_render, #lazy_builder callbacks and theme
+    // functions or templates may be used for generating a render array's
+    // content, and we might be rendering the main content for the page, it is
+    // possible that any of them throw an exception that will cause a different
+    // page to be rendered (e.g. throwing
+    // \Symfony\Component\HttpKernel\Exception\NotFoundHttpException will cause
+    // the 404 page to be rendered). That page might also use
+    // Renderer::renderRoot() but if exceptions aren't caught here, it will be
+    // impossible to call Renderer::renderRoot() again.
+    // Hence, catch all exceptions, reset the isRenderingRoot property and
+    // re-throw exceptions.
+    finally {
+      $this->isRenderingRoot = FALSE;
+    }
 
-    return $output;
+    if ((string) $output === '') {
+      return '';
+    }
+
+    return $elements['#markup'];
   }
 
   /**
    * {@inheritdoc}
    */
   public function renderInIsolation(&$elements) {
-    return $this->executeInRenderContext(new RenderContext(), function () use (&$elements) {
-      return $this->render($elements, TRUE);
-    });
+    $context = new RenderContext();
+    return Markup::create($this->executeInRenderContext($context, function () use (&$elements, $context) {
+      return $this->doRenderRoot($elements, $context);
+    }));
   }
 
   /**
@@ -223,7 +201,7 @@ class Renderer implements RendererInterface {
    * {@inheritdoc}
    */
   public function renderPlaceholder($placeholder, array $elements) {
-    // Get the render array for the given placeholder
+    // Get the render array for the given placeholder.
     $placeholder_element = $elements['#attached']['placeholders'][$placeholder];
     $markup = $this->doRenderPlaceholder($placeholder_element);
     return $this->doReplacePlaceholder($placeholder, $markup, $elements, $placeholder_element);
@@ -232,32 +210,67 @@ class Renderer implements RendererInterface {
   /**
    * {@inheritdoc}
    */
-  public function render(&$elements, $is_root_call = FALSE) {
-    // Since #pre_render, #post_render, #lazy_builder callbacks and theme
-    // functions or templates may be used for generating a render array's
-    // content, and we might be rendering the main content for the page, it is
-    // possible that any of them throw an exception that will cause a different
-    // page to be rendered (e.g. throwing
-    // \Symfony\Component\HttpKernel\Exception\NotFoundHttpException will cause
-    // the 404 page to be rendered). That page might also use
-    // Renderer::renderRoot() but if exceptions aren't caught here, it will be
-    // impossible to call Renderer::renderRoot() again.
-    // Hence, catch all exceptions, reset the isRenderingRoot property and
-    // re-throw exceptions.
-    try {
-      return $this->doRender($elements, $is_root_call);
+  public function render(/* array */&$elements, $is_root_call = FALSE) {
+
+    if (!is_array($elements)) {
+      trigger_error('Calling ' . __METHOD__ . ' with NULL is deprecated in drupal:11.3.0 and is removed from drupal:12.0.0. Either pass an array or skip the call. See https://www.drupal.org/node/3534020.');
+      return '';
     }
-    catch (\Exception $e) {
-      // Mark the ::rootRender() call finished due to this exception & re-throw.
-      $this->isRenderingRoot = FALSE;
-      throw $e;
+
+    $context = $this->getCurrentRenderContext();
+    if (!isset($context)) {
+      throw new \LogicException("Render context is empty, because render() was called outside of a renderRoot() or renderPlain() call. Use renderPlain()/renderRoot() or #lazy_builder/#pre_render instead.");
     }
+
+    if ($is_root_call) {
+      trigger_error(__METHOD__ . ' with $is_root_call is deprecated in drupal:11.2.0 and is removed from drupal:12.0.0. Use ' . __CLASS__ . '::renderRoot() instead.  See https://www.drupal.org/node/3497318.');
+      return $this->doRenderRoot($elements, $context);
+    }
+
+    return $this->doRender($elements, $context);
   }
 
   /**
    * See the docs for ::render().
    */
-  protected function doRender(&$elements, $is_root_call = FALSE) {
+  protected function doRenderRoot(array &$elements, RenderContext $context): string|MarkupInterface {
+    if (!$elements) {
+      return '';
+    }
+    // Set the bubbleable rendering metadata that has configurable defaults
+    // to ensure that the final render array definitely has these configurable
+    // defaults, even when no subtree is render cached.
+    $required_cache_contexts = $this->rendererConfig['required_cache_contexts'];
+
+    if (isset($elements['#cache']['contexts'])) {
+      $elements['#cache']['contexts'] = Cache::mergeContexts($elements['#cache']['contexts'], $required_cache_contexts);
+    }
+    else {
+      $elements['#cache']['contexts'] = $required_cache_contexts;
+    }
+
+    // Render the elements normally.
+    $return = $this->doRender($elements, $context);
+
+    // If there is no output, return early as placeholders can't make a
+    // difference.
+    if ((string) $return === '') {
+      return $return;
+    }
+
+    // Only when rendering the root do placeholders have to be processed. If we
+    // were to replace them while rendering cacheable nested elements, their
+    // cacheable metadata would still bubble all the way up the render tree,
+    // effectively making the use of placeholders pointless.
+    $this->replacePlaceholders($elements);
+
+    return $elements['#markup'];
+  }
+
+  /**
+   * See the docs for ::render().
+   */
+  protected function doRender(array &$elements, RenderContext $context): string|MarkupInterface {
     if (empty($elements)) {
       return '';
     }
@@ -278,10 +291,6 @@ class Renderer implements RendererInterface {
         $this->addCacheableDependency($elements, $elements['#access']);
         if (!$elements['#access']->isAllowed()) {
           // Abort, but bubble new cache metadata from the access result.
-          $context = $this->getCurrentRenderContext();
-          if (!isset($context)) {
-            throw new \LogicException("Render context is empty, because render() was called outside of a renderRoot() or renderInIsolation() call. Use renderInIsolation()/renderRoot() or #lazy_builder/#pre_render instead.");
-          }
           $context->push(new BubbleableMetadata());
           $context->update($elements);
           $context->bubble();
@@ -304,12 +313,10 @@ class Renderer implements RendererInterface {
     }
     $context->push(new BubbleableMetadata());
 
-    // Set the bubbleable rendering metadata that has configurable defaults, if:
-    // - this is the root call, to ensure that the final render array definitely
-    //   has these configurable defaults, even when no subtree is render cached.
-    // - this is a render cacheable subtree, to ensure that the cached data has
-    //   the configurable defaults (which may affect the ID and invalidation).
-    if ($is_root_call || isset($elements['#cache']['keys'])) {
+    // Set the bubbleable rendering metadata that has configurable defaults if
+    // this is a render cacheable subtree, to ensure that the cached data has
+    // the configurable defaults (which may affect the ID and invalidation).
+    if (isset($elements['#cache']['keys'])) {
       $required_cache_contexts = $this->rendererConfig['required_cache_contexts'];
       if (isset($elements['#cache']['contexts'])) {
         $elements['#cache']['contexts'] = Cache::mergeContexts($elements['#cache']['contexts'], $required_cache_contexts);
@@ -321,16 +328,17 @@ class Renderer implements RendererInterface {
 
     // Try to fetch the prerendered element from cache, replace any placeholders
     // and return the final markup.
-    if (isset($elements['#cache']['keys'])) {
+    //
+    // If the element was set to create a placeholder, we do not try to load it
+    // from the cache, as \Drupal\Core\Render\Placeholder\CachedStrategy has an
+    // optimization to load them all in one go, rather than individually here.
+    // If the current request method is not cacheable, no placeholders are
+    // generated, in that case, it's still better to fetch render cached
+    // elements individually instead of fully building them again.
+    if (isset($elements['#cache']['keys']) && (empty($elements['#create_placeholder']) || !$this->requestStack->getCurrentRequest()->isMethodCacheable())) {
       $cached_element = $this->renderCache->get($elements);
       if ($cached_element !== FALSE) {
         $elements = $cached_element;
-        // Only when we're in a root (non-recursive) Renderer::render() call,
-        // placeholders must be processed, to prevent breaking the render cache
-        // in case of nested elements with #cache set.
-        if ($is_root_call) {
-          $this->replacePlaceholders($elements);
-        }
         // Mark the element markup as safe if is it a string.
         if (is_string($elements['#markup'])) {
           $elements['#markup'] = Markup::create($elements['#markup']);
@@ -356,14 +364,11 @@ class Renderer implements RendererInterface {
       '#cache' => TRUE,
       '#lazy_builder' => TRUE,
       '#lazy_builder_preview' => TRUE,
+      '#placeholder_strategy_denylist' => TRUE,
       '#create_placeholder' => TRUE,
     ]);
 
-    // If the default values for this element have not been loaded yet, populate
-    // them.
-    if (isset($elements['#type']) && empty($elements['#defaults_loaded'])) {
-      $elements += $this->elementInfo->getInfo($elements['#type']);
-    }
+    $this->loadElementDefaults($elements);
 
     // First validate the usage of #lazy_builder; both of the next if-statements
     // use it if available.
@@ -380,6 +385,7 @@ class Renderer implements RendererInterface {
         '#cache',
         '#create_placeholder',
         '#lazy_builder_preview',
+        '#placeholder_strategy_denylist',
         '#preview',
         // The keys below are not actually supported, but these are added
         // automatically by the Renderer. Adding them as though they are
@@ -413,6 +419,9 @@ class Renderer implements RendererInterface {
       // provide helpful details for troubleshooting.
       assert(is_array($new_elements), "#lazy_builder callbacks must return a valid renderable array, got " . gettype($new_elements) . " from " . Variable::callableToString($elements['#lazy_builder'][0]));
 
+      // The lazy builder could have set a #type, load its defaults.
+      $this->loadElementDefaults($new_elements);
+
       // Retain the original cacheability metadata, plus cache keys.
       CacheableMetadata::createFromRenderArray($elements)
         ->merge(CacheableMetadata::createFromRenderArray($new_elements))
@@ -501,7 +510,7 @@ class Renderer implements RendererInterface {
     // same process as Renderer::render() but is inlined for speed.
     if ((!$theme_is_implemented || isset($elements['#render_children'])) && empty($elements['#children'])) {
       foreach ($children as $key) {
-        $elements['#children'] .= $this->doRender($elements[$key]);
+        $elements['#children'] .= $this->doRender($elements[$key], $context);
       }
       $elements['#children'] = Markup::create($elements['#children']);
     }
@@ -595,23 +604,6 @@ class Renderer implements RendererInterface {
       $context->update($elements);
     }
 
-    // Only when we're in a root (non-recursive) Renderer::render() call,
-    // placeholders must be processed, to prevent breaking the render cache in
-    // case of nested elements with #cache set.
-    //
-    // By running them here, we ensure that:
-    // - they run when #cache is disabled,
-    // - they run when #cache is enabled and there is a cache miss.
-    // Only the case of a cache hit when #cache is enabled, is not handled here,
-    // that is handled earlier in Renderer::render().
-    if ($is_root_call) {
-      $this->replacePlaceholders($elements);
-      // @todo remove as part of https://www.drupal.org/node/2511330.
-      if ($context->count() !== 1) {
-        throw new \LogicException('A stray RendererInterface::render() invocation with $is_root_call = TRUE is causing bubbling of attached assets to break.');
-      }
-    }
-
     // Rendering is finished, all necessary info collected!
     $context->bubble();
 
@@ -630,12 +622,40 @@ class Renderer implements RendererInterface {
    * {@inheritdoc}
    */
   public function executeInRenderContext(RenderContext $context, callable $callable) {
-    // Store the current render context.
+    // When executing in a render context, we need to isolate any bubbled
+    // context within this method. To allow for async rendering, it's necessary
+    // to detect if a fiber suspends within a render context. When this happens,
+    // we swap the previous render context in before suspending upwards, then
+    // back out again before resuming.
     $previous_context = $this->getCurrentRenderContext();
-
     // Set the provided context and call the callable, it will use that context.
     $this->setCurrentRenderContext($context);
-    $result = $callable();
+
+    $fiber = new \Fiber(static fn () => $callable());
+    $fiber->start();
+    $resume_type = NULL;
+    while (!$fiber->isTerminated()) {
+      if ($fiber->isSuspended()) {
+        // When ::executeInRenderContext() is executed within a Fiber, which is
+        // always the case when rendering placeholders, if the callback results
+        // in this fiber being suspended, we need to suspend again up to the
+        // parent Fiber. Doing so allows other placeholders to be rendered
+        // before returning here.
+        if (\Fiber::getCurrent() !== NULL) {
+          $this->setCurrentRenderContext($previous_context);
+          \Fiber::suspend();
+          $this->setCurrentRenderContext($context);
+        }
+        $resume_type = $fiber->resume();
+      }
+      // If the fiber has been suspended and has not signaled that it can be
+      // immediately resumed, assume that the fiber is waiting on an async
+      // operation and wait a bit.
+      if (!$fiber->isTerminated() && $resume_type !== FiberResumeType::Immediate) {
+        usleep(500);
+      }
+    }
+    $result = $fiber->getReturn();
     assert($context->count() <= 1, 'Bubbling failed.');
 
     // Restore the original render context.
@@ -644,6 +664,18 @@ class Renderer implements RendererInterface {
     return $result;
   }
 
+  /**
+   * Loads an element's default values based on its type.
+   *
+   * @param array $element
+   *   The render array representing the element.
+   */
+  protected function loadElementDefaults(array &$element): void {
+    if (isset($element['#type']) && empty($element['#defaults_loaded'])) {
+      $element += $this->elementInfo->getInfo($element['#type']);
+    }
+  }
+
   /**
    * Returns the current render context.
    *
@@ -652,6 +684,11 @@ class Renderer implements RendererInterface {
    */
   protected function getCurrentRenderContext() {
     $request = $this->requestStack->getCurrentRequest();
+
+    if (is_null($request)) {
+      return NULL;
+    }
+
     return static::$contextCollection[$request] ?? NULL;
   }
 
@@ -718,7 +755,7 @@ class Renderer implements RendererInterface {
         $message_placeholders[] = $placeholder;
       }
       else {
-        // Get the render array for the given placeholder
+        // Get the render array for the given placeholder.
         $fibers[$placeholder] = new \Fiber(function () use ($placeholder_element) {
           return [$this->doRenderPlaceholder($placeholder_element), $placeholder_element];
         });
@@ -777,6 +814,9 @@ class Renderer implements RendererInterface {
    * {@inheritdoc}
    */
   public function addCacheableDependency(array &$elements, $dependency) {
+    if (!$dependency instanceof CacheableDependencyInterface) {
+      @trigger_error(sprintf("Calling %s() with an object that doesn't implement %s is deprecated in drupal:11.3.0 and will throw an error in drupal:13.0.0. See https://www.drupal.org/node/3525389", __METHOD__, CacheableDependencyInterface::class), E_USER_DEPRECATED);
+    }
     $meta_a = CacheableMetadata::createFromRenderArray($elements);
     $meta_b = CacheableMetadata::createFromObject($dependency);
     $meta_a->merge($meta_b)->applyTo($elements);
