@@ -5,10 +5,12 @@ declare(strict_types=1);
 namespace Drupal\Tests\serialization\Functional;
 
 use Drupal\Tests\system\Functional\Module\GenericModuleTestBase;
+use PHPUnit\Framework\Attributes\Group;
+use PHPUnit\Framework\Attributes\RunTestsInSeparateProcesses;
 
 /**
  * Generic module test for serialization.
- *
- * @group serialization
  */
+#[Group('serialization')]
+#[RunTestsInSeparateProcesses]
 class GenericTest extends GenericModuleTestBase {}
