@@ -2,6 +2,8 @@
 
 namespace Drupal\sqlite\Driver\Database\sqlite;
 
+use Drupal\Component\Utility\FilterArray;
+use Drupal\Component\Utility\Unicode;
 use Drupal\Core\Database\Connection as DatabaseConnection;
 use Drupal\Core\Database\DatabaseNotFoundException;
 use Drupal\Core\Database\ExceptionHandler;
@@ -75,6 +77,7 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
    * Constructs a \Drupal\sqlite\Driver\Database\sqlite\Connection object.
    */
   public function __construct(\PDO $connection, array $connection_options) {
+    assert(\PHP_VERSION_ID >= 80400 ? $connection instanceof SqliteConnection : TRUE);
     parent::__construct($connection, $connection_options);
 
     // Empty prefix means query the main database -- no need to attach anything.
@@ -105,7 +108,13 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
     ];
 
     try {
-      $pdo = new PDOConnection('sqlite:' . $connection_options['database'], '', '', $connection_options['pdo']);
+      if (\PHP_VERSION_ID >= 80400) {
+        // @phpstan-ignore new.noConstructor
+        $sqlite = new SqliteConnection('sqlite:' . $connection_options['database'], '', '', $connection_options['pdo']);
+      }
+      else {
+        $sqlite = new PDOConnection('sqlite:' . $connection_options['database'], '', '', $connection_options['pdo']);
+      }
     }
     catch (\PDOException $e) {
       if ($e->getCode() == static::DATABASE_NOT_FOUND) {
@@ -117,28 +126,56 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
     }
 
     // Create functions needed by SQLite.
-    $pdo->sqliteCreateFunction('if', [__CLASS__, 'sqlFunctionIf']);
-    $pdo->sqliteCreateFunction('greatest', [__CLASS__, 'sqlFunctionGreatest']);
-    $pdo->sqliteCreateFunction('least', [__CLASS__, 'sqlFunctionLeast']);
-    $pdo->sqliteCreateFunction('pow', 'pow', 2);
-    $pdo->sqliteCreateFunction('exp', 'exp', 1);
-    $pdo->sqliteCreateFunction('length', 'strlen', 1);
-    $pdo->sqliteCreateFunction('md5', 'md5', 1);
-    $pdo->sqliteCreateFunction('concat', [__CLASS__, 'sqlFunctionConcat']);
-    $pdo->sqliteCreateFunction('concat_ws', [__CLASS__, 'sqlFunctionConcatWs']);
-    $pdo->sqliteCreateFunction('substring', [__CLASS__, 'sqlFunctionSubstring'], 3);
-    $pdo->sqliteCreateFunction('substring_index', [__CLASS__, 'sqlFunctionSubstringIndex'], 3);
-    $pdo->sqliteCreateFunction('rand', [__CLASS__, 'sqlFunctionRand']);
-    $pdo->sqliteCreateFunction('regexp', [__CLASS__, 'sqlFunctionRegexp']);
+    if (\PHP_VERSION_ID >= 80400) {
+      // Use PHP 8.4+ createFunction with fast callable syntax.
+      $sqlite->createFunction('if', self::sqlFunctionIf(...));
+      $sqlite->createFunction('greatest', self::sqlFunctionGreatest(...));
+      $sqlite->createFunction('least', self::sqlFunctionLeast(...));
+      $sqlite->createFunction('pow', pow(...), 2);
+      $sqlite->createFunction('exp', exp(...), 1);
+      $sqlite->createFunction('length', strlen(...), 1);
+      $sqlite->createFunction('md5', md5(...), 1);
+      $sqlite->createFunction('concat', self::sqlFunctionConcat(...));
+      $sqlite->createFunction('concat_ws', self::sqlFunctionConcatWs(...));
+      $sqlite->createFunction('substring', self::sqlFunctionSubstring(...), 3);
+      $sqlite->createFunction('substring_index', self::sqlFunctionSubstringIndex(...), 3);
+      $sqlite->createFunction('rand', self::sqlFunctionRand(...));
+      $sqlite->createFunction('regexp', self::sqlFunctionRegexp(...));
 
-    // SQLite does not support the LIKE BINARY operator, so we overload the
-    // non-standard GLOB operator for case-sensitive matching. Another option
-    // would have been to override another non-standard operator, MATCH, but
-    // that does not support the NOT keyword prefix.
-    $pdo->sqliteCreateFunction('glob', [__CLASS__, 'sqlFunctionLikeBinary']);
+      // SQLite does not support the LIKE BINARY operator, so we overload the
+      // non-standard GLOB operator for case-sensitive matching. Another option
+      // would have been to override another non-standard operator, MATCH, but
+      // that does not support the NOT keyword prefix.
+      $sqlite->createFunction('glob', self::sqlFunctionLikeBinary(...));
 
-    // Create a user-space case-insensitive collation with UTF-8 support.
-    $pdo->sqliteCreateCollation('NOCASE_UTF8', ['Drupal\Component\Utility\Unicode', 'strcasecmp']);
+      // Create a user-space case-insensitive collation with UTF-8 support.
+      $sqlite->createCollation('NOCASE_UTF8', Unicode::strcasecmp(...));
+    }
+    else {
+      // Fallback for PHP < 8.4
+      $sqlite->sqliteCreateFunction('if', [__CLASS__, 'sqlFunctionIf']);
+      $sqlite->sqliteCreateFunction('greatest', [__CLASS__, 'sqlFunctionGreatest']);
+      $sqlite->sqliteCreateFunction('least', [__CLASS__, 'sqlFunctionLeast']);
+      $sqlite->sqliteCreateFunction('pow', 'pow', 2);
+      $sqlite->sqliteCreateFunction('exp', 'exp', 1);
+      $sqlite->sqliteCreateFunction('length', 'strlen', 1);
+      $sqlite->sqliteCreateFunction('md5', 'md5', 1);
+      $sqlite->sqliteCreateFunction('concat', [__CLASS__, 'sqlFunctionConcat']);
+      $sqlite->sqliteCreateFunction('concat_ws', [__CLASS__, 'sqlFunctionConcatWs']);
+      $sqlite->sqliteCreateFunction('substring', [__CLASS__, 'sqlFunctionSubstring'], 3);
+      $sqlite->sqliteCreateFunction('substring_index', [__CLASS__, 'sqlFunctionSubstringIndex'], 3);
+      $sqlite->sqliteCreateFunction('rand', [__CLASS__, 'sqlFunctionRand']);
+      $sqlite->sqliteCreateFunction('regexp', [__CLASS__, 'sqlFunctionRegexp']);
+
+      // SQLite does not support the LIKE BINARY operator, so we overload the
+      // non-standard GLOB operator for case-sensitive matching. Another option
+      // would have been to override another non-standard operator, MATCH, but
+      // that does not support the NOT keyword prefix.
+      $sqlite->sqliteCreateFunction('glob', [__CLASS__, 'sqlFunctionLikeBinary']);
+
+      // Create a user-space case-insensitive collation with UTF-8 support.
+      $sqlite->sqliteCreateCollation('NOCASE_UTF8', ['Drupal\Component\Utility\Unicode', 'strcasecmp']);
+    }
 
     // Set SQLite init_commands if not already defined. Enable the Write-Ahead
     // Logging (WAL) for SQLite. See https://www.drupal.org/node/2348137 and
@@ -152,10 +189,10 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
 
     // Execute sqlite init_commands.
     if (isset($connection_options['init_commands'])) {
-      $pdo->exec(implode('; ', $connection_options['init_commands']));
+      $sqlite->exec(implode('; ', $connection_options['init_commands']));
     }
 
-    return $pdo;
+    return $sqlite;
   }
 
   /**
@@ -168,9 +205,13 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
   public function __destruct() {
     if ($this->tableDropped && !empty($this->attachedDatabases)) {
       foreach ($this->attachedDatabases as $prefix) {
-        // Check if the database is now empty, ignore the internal SQLite tables.
+        // Check if the database is now empty, ignore the internal SQLite
+        // tables.
         try {
-          $count = $this->query('SELECT COUNT(*) FROM ' . $prefix . '.sqlite_master WHERE type = :type AND name NOT LIKE :pattern', [':type' => 'table', ':pattern' => 'sqlite_%'])->fetchField();
+          $count = $this->query('SELECT COUNT(*) FROM ' . $prefix . '.sqlite_master WHERE type = :type AND name NOT LIKE :pattern', [
+            ':type' => 'table',
+            ':pattern' => 'sqlite_%',
+          ])->fetchField();
 
           // We can prune the database file if it doesn't have any tables.
           if ($count == 0 && $this->connectionOptions['database'] != ':memory:' && file_exists($this->connectionOptions['database'] . '-' . $prefix)) {
@@ -180,7 +221,7 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
             unlink($this->connectionOptions['database'] . '-' . $prefix);
           }
         }
-        catch (\Exception $e) {
+        catch (\Exception) {
           // Ignore the exception and continue. There is nothing we can do here
           // to report the error or fail safe.
         }
@@ -199,7 +240,10 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
       // http://www.sqlite.org/inmemorydb.html it will open a unique database so
       // attaching it twice is not a problem.
       $database_file = $this->connectionOptions['database'] !== ':memory:' ? $this->connectionOptions['database'] . '-' . $database : $this->connectionOptions['database'];
-      $this->query('ATTACH DATABASE :database_file AS :database', [':database_file' => $database_file, ':database' => $database]);
+      $this->query('ATTACH DATABASE :database_file AS :database', [
+        ':database_file' => $database_file,
+        ':database' => $database,
+      ]);
       $this->attachedDatabases[$database] = $database;
     }
   }
@@ -245,8 +289,9 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
    * SQLite compatibility implementation for the LEAST() SQL function.
    */
   public static function sqlFunctionLeast() {
-    // Remove all NULL, FALSE and empty strings values but leaves 0 (zero) values.
-    $values = array_filter(func_get_args(), 'strlen');
+    // Remove all NULL, FALSE and empty strings values but leaves 0 (zero)
+    // values.
+    $values = FilterArray::removeEmptyStrings(func_get_args());
 
     return count($values) < 1 ? NULL : min($values);
   }
@@ -346,6 +391,9 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
     return preg_match('/^' . $pattern . '$/', $subject);
   }
 
+  /**
+   * {@inheritdoc}
+   */
   public function queryRange($query, $from, $count, array $args = [], array $options = []) {
     return $this->query($query . ' LIMIT ' . (int) $from . ', ' . (int) $count, $args, $options);
   }
@@ -366,10 +414,16 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
     return 'temp.' . $tablename;
   }
 
+  /**
+   * {@inheritdoc}
+   */
   public function driver() {
     return 'sqlite';
   }
 
+  /**
+   * {@inheritdoc}
+   */
   public function databaseType() {
     return 'sqlite';
   }
@@ -390,6 +444,9 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
     }
   }
 
+  /**
+   * {@inheritdoc}
+   */
   public function mapConditionOperator($operator) {
     return static::$sqliteConditionOperatorMap[$operator] ?? NULL;
   }
@@ -399,6 +456,9 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
    */
   public function prepareStatement(string $query, array $options, bool $allow_row_count = FALSE): StatementInterface {
     assert(!isset($options['return']), 'Passing "return" option to prepareStatement() has no effect. See https://www.drupal.org/node/3185520');
+    if (isset($options['fetch']) && is_int($options['fetch'])) {
+      @trigger_error("Passing the 'fetch' key as an integer to \$options in prepareStatement() is deprecated in drupal:11.2.0 and is removed from drupal:12.0.0. Use a case of \Drupal\Core\Database\Statement\FetchAs enum instead. See https://www.drupal.org/node/3488338", E_USER_DEPRECATED);
+    }
 
     try {
       $query = $this->preprocessStatement($query, $options);
@@ -424,7 +484,10 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
    * {@inheritdoc}
    */
   public static function createConnectionOptionsFromUrl($url, $root) {
-    $database = parent::createConnectionOptionsFromUrl($url, $root);
+    if ($root !== NULL) {
+      @trigger_error("Passing the \$root value to " . __METHOD__ . "() is deprecated in drupal:11.2.0 and will be removed in drupal:12.0.0. There is no replacement. See https://www.drupal.org/node/3511287", E_USER_DEPRECATED);
+    }
+    $database = parent::createConnectionOptionsFromUrl($url, NULL);
 
     // A SQLite database path with two leading slashes indicates a system path.
     // Otherwise the path is relative to the Drupal root.
@@ -432,12 +495,7 @@ class Connection extends DatabaseConnection implements SupportsTemporaryTablesIn
     if ($url_components['path'][0] === '/') {
       $url_components['path'] = substr($url_components['path'], 1);
     }
-    if ($url_components['path'][0] === '/' || $url_components['path'] === ':memory:') {
-      $database['database'] = $url_components['path'];
-    }
-    else {
-      $database['database'] = $root . '/' . $url_components['path'];
-    }
+    $database['database'] = $url_components['path'];
 
     // User credentials and system port are irrelevant for SQLite.
     unset(
